#!/usr/bin/env python3

from bunkai.algorithm.bunkai_sbd.bunkai_sbd import BunkaiSentenceBoundaryDisambiguation as Bunkai

__all__ = ["Bunkai"]
__version_info__ = (1, 5, 7)
__version__ = ".".join(map(str, __version_info__))
